/**** SimCore since 2002-05-15 by Kenji KISE ****/ 
/************************************************/
#include "define.h"

/************************************************/
simple_chip::simple_chip(char *prog, char **opt){
  ev      = new Env();
  ev->sc  = new system_config(prog, opt);
  ev->e   = new evaluation_result();
  ev->as  = new architecture_state(ev);
  ev->mem = new memory_system(ev);
  ev->deb = new debug(ev);
  ev->sys = new system_manager(ev);
  p       = new instruction(ev);

  if(ev->sc->distrib_flag) 
    ev->e->imix = new main_memory();
}

/************************************************/
simple_chip::~simple_chip(){   /** destructor **/
  printf("\n");
  for(int i=0; i<64; i++) printf("="); 
  printf("\n=== %s/%s %s\n", PROJ, NAME, VER);
  print_evaluation_result(ev);
  if(ev->sc->result_flag){
    data_t a, d;
    while(ev->mem->mm->load_all(&a, &d)){
      if(a>=0x40000000)
	printf("1%llx %16llx\n", a, d);
    }
  }
}

/************************************************/
int simple_chip::step(){
  p->Fetch(&ev->as->pc); /* pipeline stage 0 */
  p->Slot();             /* pipeline stage 1 */
  p->Issue();            /* pipeline stage 3 */
  p->RegisterRead();     /* pipeline stage 4 */
  p->Execute();          /* pipeline stage 5 */
  p->Memory();           /* pipeline stage 6 */
  p->WriteBack();
  
  ev->e->retired_inst++;
  house_keeper(p);
  
  return ev->sys->running;
}

#define IMSK 0x0ffff  /* mask of inst_buf */
/** main loop of the optimized version         **/
/************************************************/
void simple_chip::loop(){
  instruction **ib = new instruction*[IMSK+1];
  for(int i=0; i<IMSK+1; i++){
    ib[i] = new instruction(ev);
  }
  
  while(ev->sys->running){
    int index = (ev->as->pc>>2) & IMSK;
    instruction *pt = ib[index];
    
    if(pt->Cpc!=ev->as->pc){
      pt->Fetch(&ev->as->pc);
      pt->Slot();
      pt->Issue();
    }
    pt->BackEnd();

    ev->e->retired_inst++;
    if(ev->sc->slow_mode) house_keeper(pt);
   }
}
/************************************************/
